@@ -2,7 +2,7 @@
 //
 // based on Daniel Grunwald's https://github.com/dgrunwald/rust-cpython
 
-use crate::{ffi, AsPyPointer, PyAny, PyErr, PyResult, Python};
+use crate::{ffi, AsPyPointer, IntoPyPointer, Py, PyAny, PyErr, PyNativeType, PyResult, Python};
 #[cfg(any(not(Py_LIMITED_API), Py_3_8))]
 use crate::{PyDowncastError, PyTryFrom};
 
@@ -58,13 +58,7 @@ impl<'p> Iterator for &'p PyIterator {
 
         match unsafe { py.from_owned_ptr_or_opt(ffi::PyIter_Next(self.0.as_ptr())) } {
             Some(obj) => Some(Ok(obj)),
-            None => {
-                if PyErr::occurred(py) {
-                    Some(Err(PyErr::fetch(py)))
-                } else {
-                    None
-                }
-            }
+            None => PyErr::take(py).map(Err),
         }
     }
 }
@@ -94,6 +88,22 @@ impl<'v> PyTryFrom<'v> for PyIterator {
     }
 }
 
+impl Py<PyIterator> {
+    /// Borrows a GIL-bound reference to the PyIterator. By binding to the GIL lifetime, this
+    /// allows the GIL-bound reference to not require `Python` for any of its methods.
+    pub fn as_ref<'py>(&'py self, _py: Python<'py>) -> &'py PyIterator {
+        let any = self.as_ptr() as *const PyAny;
+        unsafe { PyNativeType::unchecked_downcast(&*any) }
+    }
+
+    /// Similar to [`as_ref`](#method.as_ref), and also consumes this `Py` and registers the
+    /// Python object reference in PyO3's object storage. The reference count for the Python
+    /// object will not be decreased until the GIL lifetime ends.
+    pub fn into_ref(self, py: Python) -> &PyIterator {
+        unsafe { py.from_owned_ptr(self.into_ptr()) }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::PyIterator;
@@ -101,9 +111,8 @@ mod tests {
     use crate::gil::GILPool;
     use crate::types::{PyDict, PyList};
     #[cfg(any(not(Py_LIMITED_API), Py_3_8))]
-    use crate::{Py, PyAny, PyTryFrom};
-    use crate::{Python, ToPyObject};
-    use indoc::indoc;
+    use crate::PyTryFrom;
+    use crate::{Py, PyAny, Python, ToPyObject};
 
     #[test]
     fn vec_iter() {
@@ -167,16 +176,14 @@ mod tests {
 
     #[test]
     fn fibonacci_generator() {
-        let fibonacci_generator = indoc!(
-            r#"
-            def fibonacci(target):
-                a = 1
-                b = 1
-                for _ in range(target):
-                    yield a
-                    a, b = b, a + b
-        "#
-        );
+        let fibonacci_generator = r#"
+def fibonacci(target):
+    a = 1
+    b = 1
+    for _ in range(target):
+        yield a
+        a, b = b, a + b
+"#;
 
         Python::with_gil(|py| {
             let context = PyDict::new(py);
@@ -196,7 +203,7 @@ mod tests {
             let x = 5.to_object(py);
             let err = PyIterator::from_object(py, &x).unwrap_err();
 
-            assert!(err.is_instance::<PyTypeError>(py));
+            assert!(err.is_instance_of::<PyTypeError>(py));
         });
     }
 
@@ -206,7 +213,29 @@ mod tests {
         Python::with_gil(|py| {
             let obj: Py<PyAny> = vec![10, 20].to_object(py).as_ref(py).iter().unwrap().into();
             let iter: &PyIterator = PyIterator::try_from(obj.as_ref(py)).unwrap();
-            assert_eq!(obj, iter.into());
+            assert!(obj.is(iter));
         });
     }
+
+    #[test]
+    fn test_as_ref() {
+        Python::with_gil(|py| {
+            let iter: Py<PyIterator> = PyAny::iter(PyList::empty(py)).unwrap().into();
+            let mut iter_ref: &PyIterator = iter.as_ref(py);
+            assert!(iter_ref.next().is_none());
+        })
+    }
+
+    #[test]
+    fn test_into_ref() {
+        Python::with_gil(|py| {
+            let bare_iter = PyAny::iter(PyList::empty(py)).unwrap();
+            assert_eq!(bare_iter.get_refcnt(), 1);
+            let iter: Py<PyIterator> = bare_iter.into();
+            assert_eq!(bare_iter.get_refcnt(), 2);
+            let mut iter_ref = iter.into_ref(py);
+            assert!(iter_ref.next().is_none());
+            assert_eq!(iter_ref.get_refcnt(), 2);
+        })
+    }
 }
