@@ -1,10 +1,8 @@
-#![allow(deprecated)] // for deprecated protocol methods
+#![cfg(feature = "macros")]
 
 use pyo3::class::basic::CompareOp;
-use pyo3::class::*;
 use pyo3::prelude::*;
 use pyo3::py_run;
-use pyo3::PyNativeType;
 
 mod common;
 
@@ -13,21 +11,17 @@ struct UnaryArithmetic {
     inner: f64,
 }
 
+#[pymethods]
 impl UnaryArithmetic {
+    #[new]
     fn new(value: f64) -> Self {
         UnaryArithmetic { inner: value }
     }
-}
 
-#[pyproto]
-impl PyObjectProtocol for UnaryArithmetic {
     fn __repr__(&self) -> String {
         format!("UA({})", self.inner)
     }
-}
 
-#[pyproto]
-impl PyNumberProtocol for UnaryArithmetic {
     fn __neg__(&self) -> Self {
         Self::new(-self.inner)
     }
@@ -59,13 +53,36 @@ fn unary_arithmetic() {
 }
 
 #[pyclass]
-struct BinaryArithmetic {}
+struct Indexable(i32);
 
-#[pyproto]
-impl PyObjectProtocol for BinaryArithmetic {
-    fn __repr__(&self) -> &'static str {
-        "BA"
+#[pymethods]
+impl Indexable {
+    fn __index__(&self) -> i32 {
+        self.0
     }
+
+    fn __int__(&self) -> i32 {
+        self.0
+    }
+
+    fn __float__(&self) -> f64 {
+        f64::from(self.0)
+    }
+
+    fn __invert__(&self) -> Self {
+        Self(!self.0)
+    }
+}
+
+#[test]
+fn indexable() {
+    Python::with_gil(|py| {
+        let i = PyCell::new(py, Indexable(5)).unwrap();
+        py_run!(py, i, "assert int(i) == 5");
+        py_run!(py, i, "assert [0, 1, 2, 3, 4, 5][i] == 5");
+        py_run!(py, i, "assert float(i) == 5.0");
+        py_run!(py, i, "assert int(~i) == -6");
+    })
 }
 
 #[pyclass]
@@ -73,15 +90,12 @@ struct InPlaceOperations {
     value: u32,
 }
 
-#[pyproto]
-impl PyObjectProtocol for InPlaceOperations {
+#[pymethods]
+impl InPlaceOperations {
     fn __repr__(&self) -> String {
         format!("IPO({:?})", self.value)
     }
-}
 
-#[pyproto]
-impl PyNumberProtocol for InPlaceOperations {
     fn __iadd__(&mut self, other: u32) {
         self.value += other;
     }
@@ -114,7 +128,7 @@ impl PyNumberProtocol for InPlaceOperations {
         self.value |= other;
     }
 
-    fn __ipow__(&mut self, other: u32) {
+    fn __ipow__(&mut self, other: u32, _modulo: Option<u32>) {
         self.value = self.value.pow(other);
     }
 }
@@ -143,42 +157,49 @@ fn inplace_operations() {
     );
 }
 
-#[pyproto]
-impl PyNumberProtocol for BinaryArithmetic {
-    fn __add__(lhs: &PyAny, rhs: &PyAny) -> String {
-        format!("{:?} + {:?}", lhs, rhs)
+#[pyclass]
+struct BinaryArithmetic {}
+
+#[pymethods]
+impl BinaryArithmetic {
+    fn __repr__(&self) -> &'static str {
+        "BA"
     }
 
-    fn __sub__(lhs: &PyAny, rhs: &PyAny) -> String {
-        format!("{:?} - {:?}", lhs, rhs)
+    fn __add__(&self, rhs: &PyAny) -> String {
+        format!("BA + {:?}", rhs)
     }
 
-    fn __mul__(lhs: &PyAny, rhs: &PyAny) -> String {
-        format!("{:?} * {:?}", lhs, rhs)
+    fn __sub__(&self, rhs: &PyAny) -> String {
+        format!("BA - {:?}", rhs)
     }
 
-    fn __lshift__(lhs: &PyAny, rhs: &PyAny) -> String {
-        format!("{:?} << {:?}", lhs, rhs)
+    fn __mul__(&self, rhs: &PyAny) -> String {
+        format!("BA * {:?}", rhs)
     }
 
-    fn __rshift__(lhs: &PyAny, rhs: &PyAny) -> String {
-        format!("{:?} >> {:?}", lhs, rhs)
+    fn __lshift__(&self, rhs: &PyAny) -> String {
+        format!("BA << {:?}", rhs)
     }
 
-    fn __and__(lhs: &PyAny, rhs: &PyAny) -> String {
-        format!("{:?} & {:?}", lhs, rhs)
+    fn __rshift__(&self, rhs: &PyAny) -> String {
+        format!("BA >> {:?}", rhs)
     }
 
-    fn __xor__(lhs: &PyAny, rhs: &PyAny) -> String {
-        format!("{:?} ^ {:?}", lhs, rhs)
+    fn __and__(&self, rhs: &PyAny) -> String {
+        format!("BA & {:?}", rhs)
     }
 
-    fn __or__(lhs: &PyAny, rhs: &PyAny) -> String {
-        format!("{:?} | {:?}", lhs, rhs)
+    fn __xor__(&self, rhs: &PyAny) -> String {
+        format!("BA ^ {:?}", rhs)
     }
 
-    fn __pow__(lhs: &PyAny, rhs: &PyAny, mod_: Option<u32>) -> String {
-        format!("{:?} ** {:?} (mod: {:?})", lhs, rhs, mod_)
+    fn __or__(&self, rhs: &PyAny) -> String {
+        format!("BA | {:?}", rhs)
+    }
+
+    fn __pow__(&self, rhs: &PyAny, mod_: Option<u32>) -> String {
+        format!("BA ** {:?} (mod: {:?})", rhs, mod_)
     }
 }
 
@@ -191,24 +212,27 @@ fn binary_arithmetic() {
     py_run!(py, c, "assert c + c == 'BA + BA'");
     py_run!(py, c, "assert c.__add__(c) == 'BA + BA'");
     py_run!(py, c, "assert c + 1 == 'BA + 1'");
-    py_run!(py, c, "assert 1 + c == '1 + BA'");
     py_run!(py, c, "assert c - 1 == 'BA - 1'");
-    py_run!(py, c, "assert 1 - c == '1 - BA'");
     py_run!(py, c, "assert c * 1 == 'BA * 1'");
-    py_run!(py, c, "assert 1 * c == '1 * BA'");
-
     py_run!(py, c, "assert c << 1 == 'BA << 1'");
-    py_run!(py, c, "assert 1 << c == '1 << BA'");
     py_run!(py, c, "assert c >> 1 == 'BA >> 1'");
-    py_run!(py, c, "assert 1 >> c == '1 >> BA'");
     py_run!(py, c, "assert c & 1 == 'BA & 1'");
-    py_run!(py, c, "assert 1 & c == '1 & BA'");
     py_run!(py, c, "assert c ^ 1 == 'BA ^ 1'");
-    py_run!(py, c, "assert 1 ^ c == '1 ^ BA'");
     py_run!(py, c, "assert c | 1 == 'BA | 1'");
-    py_run!(py, c, "assert 1 | c == '1 | BA'");
     py_run!(py, c, "assert c ** 1 == 'BA ** 1 (mod: None)'");
-    py_run!(py, c, "assert 1 ** c == '1 ** BA (mod: None)'");
+
+    // Class with __add__ only should not allow the reverse op;
+    // this is consistent with Python classes.
+
+    py_expect_exception!(py, c, "1 + c", PyTypeError);
+    py_expect_exception!(py, c, "1 - c", PyTypeError);
+    py_expect_exception!(py, c, "1 * c", PyTypeError);
+    py_expect_exception!(py, c, "1 << c", PyTypeError);
+    py_expect_exception!(py, c, "1 >> c", PyTypeError);
+    py_expect_exception!(py, c, "1 & c", PyTypeError);
+    py_expect_exception!(py, c, "1 ^ c", PyTypeError);
+    py_expect_exception!(py, c, "1 | c", PyTypeError);
+    py_expect_exception!(py, c, "1 ** c", PyTypeError);
 
     py_run!(py, c, "assert pow(c, 1, 100) == 'BA ** 1 (mod: Some(100))'");
 }
@@ -216,8 +240,8 @@ fn binary_arithmetic() {
 #[pyclass]
 struct RhsArithmetic {}
 
-#[pyproto]
-impl PyNumberProtocol for RhsArithmetic {
+#[pymethods]
+impl RhsArithmetic {
     fn __radd__(&self, other: &PyAny) -> String {
         format!("{:?} + RA", other)
     }
@@ -250,7 +274,7 @@ impl PyNumberProtocol for RhsArithmetic {
         format!("{:?} | RA", other)
     }
 
-    fn __rpow__(&self, other: &PyAny, _mod: Option<&'p PyAny>) -> String {
+    fn __rpow__(&self, other: &PyAny, _mod: Option<&PyAny>) -> String {
         format!("{:?} ** RA", other)
     }
 }
@@ -290,8 +314,12 @@ impl std::fmt::Debug for LhsAndRhs {
     }
 }
 
-#[pyproto]
-impl PyNumberProtocol for LhsAndRhs {
+#[pymethods]
+impl LhsAndRhs {
+    // fn __repr__(&self) -> &'static str {
+    //     "BA"
+    // }
+
     fn __add__(lhs: PyRef<Self>, rhs: &PyAny) -> String {
         format!("{:?} + {:?}", lhs, rhs)
     }
@@ -364,19 +392,20 @@ impl PyNumberProtocol for LhsAndRhs {
         format!("{:?} | RA", other)
     }
 
-    fn __rpow__(&self, other: &PyAny, _mod: Option<&'p PyAny>) -> String {
+    fn __rpow__(&self, other: &PyAny, _mod: Option<&PyAny>) -> String {
         format!("{:?} ** RA", other)
     }
 
     fn __rmatmul__(&self, other: &PyAny) -> String {
         format!("{:?} @ RA", other)
     }
-}
 
-#[pyproto]
-impl PyObjectProtocol for LhsAndRhs {
-    fn __repr__(&self) -> &'static str {
-        "BA"
+    fn __rtruediv__(&self, other: &PyAny) -> String {
+        format!("{:?} / RA", other)
+    }
+
+    fn __rfloordiv__(&self, other: &PyAny) -> String {
+        format!("{:?} // RA", other)
     }
 }
 
@@ -413,8 +442,8 @@ fn lhs_fellback_to_rhs() {
 #[pyclass]
 struct RichComparisons {}
 
-#[pyproto]
-impl PyObjectProtocol for RichComparisons {
+#[pymethods]
+impl RichComparisons {
     fn __repr__(&self) -> &'static str {
         "RC"
     }
@@ -434,8 +463,8 @@ impl PyObjectProtocol for RichComparisons {
 #[pyclass]
 struct RichComparisons2 {}
 
-#[pyproto]
-impl PyObjectProtocol for RichComparisons2 {
+#[pymethods]
+impl RichComparisons2 {
     fn __repr__(&self) -> &'static str {
         "RC2"
     }
@@ -509,76 +538,73 @@ mod return_not_implemented {
     #[pyclass]
     struct RichComparisonToSelf {}
 
-    #[pyproto]
-    impl<'p> PyObjectProtocol<'p> for RichComparisonToSelf {
+    #[pymethods]
+    impl RichComparisonToSelf {
         fn __repr__(&self) -> &'static str {
             "RC_Self"
         }
 
-        fn __richcmp__(&self, other: PyRef<'p, Self>, _op: CompareOp) -> PyObject {
+        fn __richcmp__(&self, other: PyRef<Self>, _op: CompareOp) -> PyObject {
             other.py().None()
         }
-    }
 
-    #[pyproto]
-    impl<'p> PyNumberProtocol<'p> for RichComparisonToSelf {
-        fn __add__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __add__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __sub__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __sub__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __mul__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __mul__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __matmul__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __matmul__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __truediv__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __truediv__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __floordiv__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __floordiv__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __mod__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __mod__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __pow__(lhs: &'p PyAny, _other: u8, _modulo: Option<u8>) -> &'p PyAny {
-            lhs
+        fn __pow__(slf: PyRef<Self>, _other: u8, _modulo: Option<u8>) -> PyRef<Self> {
+            slf
         }
-        fn __lshift__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __lshift__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __rshift__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __rshift__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __divmod__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __divmod__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __and__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __and__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __or__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __or__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
-        fn __xor__(lhs: &'p PyAny, _other: PyRef<'p, Self>) -> &'p PyAny {
-            lhs
+        fn __xor__<'p>(slf: PyRef<'p, Self>, _other: PyRef<'p, Self>) -> PyRef<'p, Self> {
+            slf
         }
 
         // Inplace assignments
-        fn __iadd__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __isub__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __imul__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __imatmul__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __itruediv__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __ifloordiv__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __imod__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __ipow__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __ilshift__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __irshift__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __iand__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __ior__(&'p mut self, _other: PyRef<'p, Self>) {}
-        fn __ixor__(&'p mut self, _other: PyRef<'p, Self>) {}
+        fn __iadd__(&mut self, _other: PyRef<Self>) {}
+        fn __isub__(&mut self, _other: PyRef<Self>) {}
+        fn __imul__(&mut self, _other: PyRef<Self>) {}
+        fn __imatmul__(&mut self, _other: PyRef<Self>) {}
+        fn __itruediv__(&mut self, _other: PyRef<Self>) {}
+        fn __ifloordiv__(&mut self, _other: PyRef<Self>) {}
+        fn __imod__(&mut self, _other: PyRef<Self>) {}
+        fn __ilshift__(&mut self, _other: PyRef<Self>) {}
+        fn __irshift__(&mut self, _other: PyRef<Self>) {}
+        fn __iand__(&mut self, _other: PyRef<Self>) {}
+        fn __ior__(&mut self, _other: PyRef<Self>) {}
+        fn __ixor__(&mut self, _other: PyRef<Self>) {}
+        fn __ipow__(&mut self, _other: PyRef<Self>, _modulo: Option<u8>) {}
     }
 
     fn _test_binary_dunder(dunder: &str) {
@@ -649,7 +675,6 @@ mod return_not_implemented {
     }
 
     #[test]
-    #[ignore]
     fn reverse_arith() {
         _test_binary_dunder("radd");
         _test_binary_dunder("rsub");
@@ -658,6 +683,7 @@ mod return_not_implemented {
         _test_binary_dunder("rtruediv");
         _test_binary_dunder("rfloordiv");
         _test_binary_dunder("rmod");
+        _test_binary_dunder("rdivmod");
         _test_binary_dunder("rpow");
     }
 
